function [sol iter resvec] = multi_grid( b, nu ,show_err,maxiter,x0)
%performs a mult-grid method for Ax = b
% A discrete Laplacian on N x N grid, N = 2^p-1
% nu is the number of pre-smoothing steps
% b given as N x N array
% the recursive calls are in mg.m
if nargin<3 || isempty(show_err),
   show_err=0;
end
if nargin<4 || isempty(maxiter),
   maxiter=Inf;
end
if size(b,2)==1,
   n2=size(b,1);
   n=sqrt(n2);
   b=reshape(b,n,n);
end
eps = 1e-10;
n = size(b,1);
nc = floor(n/2);
Ac = laplace(nc);
if nargin<5,
   u = rand(n);
else
   u = x0;
end
r = b-Laplace_mult(u);
resvec(1)=norm(r);
if show_err,
   A=laplace(n);
   sol=A\b(:);
   sol=reshape(sol,n,n);
end
iter = 0;
while (norm(r,'fro') > eps && iter<maxiter)  
      if show_err,
         figure(show_err)
         mesh(reshape(u-sol,n,n));
         title('error for before smoothing')
	 pause(1)
      end

      [e,r] = smoothing(zeros(n),r,nu);
      u = u+e;

      if show_err,
         figure(show_err)
         mesh(reshape(u-sol,n,n));
         title('error for before coarse grid correction')
	 pause(1)
      end

      rc = restriction(r);
      solc = mg(rc,nu);
      u = u + prolongation(solc);
      r = b - Laplace_mult(u);
      iter = iter+1;
      
      resvec(iter+1)=norm(r);
end
sol = u;
%iter

end

